/**************************************************************** 
 * Description: 
 * Author: Alex Li
 * Date: 2024-08-10 15:50:56
 * LastEditTime: 2024-08-19 00:14:50
****************************************************************/
#include <iostream>
#include <set>

using namespace std;

// 定义一个 multiset 来存储元素，multiset 允许存储重复元素
multiset<int> s;

// 插入一个数 x
void insert(int x) {
    s.insert(x);
}

// 删除一个数 x（若有多个相同的数，应只删除一个）
void erase(int x) {
    // 使用 find 查找元素 x 的位置
    auto it = s.find(x);
    // 如果找到了元素，则删除它
    if (it != s.end()) {
        s.erase(it);
    }
}

// 查询 x 的排名（比 x 小的数的个数 + 1）
int getRank(int x) {
    // 使用 lower_bound 查找第一个不小于 x 的元素的位置
    return distance(s.begin(), s.lower_bound(x)) + 1;
}

// 查询排名为 x 的数
int getSelect(int x) {
    // 从集合的头部开始，移动到第 x-1 个元素的位置
    auto it = s.begin();
    advance(it, x - 1);
    return *it;
}

// 求 x 的前驱（前驱定义为小于 x，且最大的数）
int getPredecessor(int x) {
    // 使用 lower_bound 查找第一个不小于 x 的元素
    auto it = s.lower_bound(x);
    // 如果 x 是集合中的最小值，则没有前驱
    if (it == s.begin()) return -1;
    // 否则返回前一个元素
    it--;
    return *it;
}

// 求 x 的后继（后继定义为大于 x，且最小的数）
int getSuccessor(int x) {
    // 使用 upper_bound 查找第一个大于 x 的元素
    auto it = s.upper_bound(x);
    // 如果 x 是集合中的最大值，则没有后继
    if (it == s.end()) return -1;
    return *it;
}

int main() {
    int n;
    // 读取操作的个数 n
    cin >> n;

    while (n--) {
        int opt, x;
        // 读取操作类型 opt 和元素 x
        cin >> opt >> x;

        // 根据 opt 的值执行不同的操作
        if (opt == 1) {
            insert(x);  // 插入操作
        } else if (opt == 2) {
            erase(x);  // 删除操作
        } else if (opt == 3) {
            cout << getRank(x) << endl;  // 查询 x 的排名
        } else if (opt == 4) {
            cout << getSelect(x) << endl;  // 查询排名为 x 的数
        } else if (opt == 5) {
            cout << getPredecessor(x) << endl;  // 查询 x 的前驱
        } else if (opt == 6) {
            cout << getSuccessor(x) << endl;  // 查询 x 的后继
        }
    }

    return 0;
}
